@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
-# Copyright 2009-2016 Yelp and Contributors
+# Copyright 2009-2018 Yelp and Contributors
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -14,9 +14,8 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-"""Write and run Hadoop Streaming jobs on Amazon Elastic MapReduce or your own
-Hadoop cluster.
-"""
+"""Write and run Hadoop Streaming jobs on
+Amazon Elastic MapReduce / Google Dataproc / your own Hadoop cluster."""
 
 __author__ = 'David Marin <dm@davidmarin.org>'
 
@@ -25,6 +24,9 @@ __credits__ = [
     'Jordan Andersen <jordandandersen@gmail.com>',
     'Tom Arnfeld <tarnfeld@me.com>',
     'Martin Baeuml <baeuml@kit.edu>',
+    'Benjamin Bariteau <bariteau@yelp.com>',
+    'Étienne Batise <etienne.batise@gmail.com>',
+    'Edward Betts <edward@4angle.com>',
     'Hunter Blanks <hblanks@monetate.com>',
     'Jim Blomo <jblomo@yelp.com>',
     'Reno Bowen <renobowen@gmail.com>',
@@ -43,8 +45,10 @@ __credits__ = [
     'Tomer Elmalem <telmalem@gmail.com>',
     'Pedro Emanuel de Castro Faria Salgado <steenzout@ymail.com>',
     'Buck Evan <buck@yelp.com>',
+    'Cesar Gonzalez Fernandez <cesar.gon.fer@gmail.com>',
     'Dan Frank <danielhfrank@gmail.com>',
     'Sudarshan Gaikaiwari <sudarshan@acm.org>',
+    'Alex Gaynor <alex.gaynor@gmail.com>',
     'Brett Gibson <brett@swiftserve.com>',
     'Benjamin Goldenberg <benjamin@yelp.com',
     'Thomas Grainger <tagrain@gmail.com>',
@@ -54,8 +58,8 @@ __credits__ = [
     'Tom Hennigan <tomhennigan@gmail.com>',
     'Brett Hoerner <brett@bretthoerner.com>',
     'Ya-Lin Huang <yalinh@yelp.com>',
-    'Evan Klitzke <evan@eklitzke.org>',
     'Tom Janofsky <tjanofsky@monetate.com>',
+    'Tim Jenks <tim.jenks@eegeo.com>',
     'Stephen Johnson <steve@steveasleep.com>',
     'Ben Jolitz <Ben.Jolitz.Acxiom.com>',
     'Matt Jones <matt@mhjones.org>',
@@ -67,11 +71,15 @@ __credits__ = [
     'Julian Krause <juliank@yelp.com>',
     'Pai-Wei Lai <paiwei@yelp.com>',
     'Boris Lau <boris.w.lau@gmail.com>',
+    'Jerome Leclanche <jerome@leclan.ch>',
     'Robert Leftwich <rl.0x0@eml.cc>',
     'Andrew Lenards <andrew.lenards@gmail.com>',
+    'Ronny Lindner <ronny.lindner@invia.de>',
     'Tianhui Michael Li <tianhuil@cs.princeton.edu>',
+    'James Lim <jim@affirm.com>',
     'Shusen Liu <liushusen.smart@gmail.com>',
     'Adrian Maceiras <amac425@utexas.edu>',
+    'Chet Mancini <chetmancini@gmail.com>',
     'Dallas Marlow <dallas@vineapp.com>',
     'Colin Matthias <cmatthias@cogolabs.com>',
     'Baris Metin <bmetin@yelp.com>',
@@ -86,9 +94,11 @@ __credits__ = [
     'Dávid Nemeskey <david@cliqz.com>',
     'Daniel Nephin <dnephin@yelp.com>',
     'Oliver Nicholas <bigo@wonlove.net>',
+    'Tyler Nisonoff <tyler@asana.com>',
     'Santeri Paavolainen <santtu@iki.fi>',
     'Matt Perry <matt@unshift.net>',
     'Kien Pham <kien@sendgrid.com>',
+    'Adam Phillabaum <adam@phillabaum.us>',
     'Andrew Price <andrew.price@ensighten.com>',
     'Wahbeh Qardaji <wahbeh.qardaji@gmail.com>',
     'Anusha Rajan <anusha@yelp.com>',
@@ -111,10 +121,13 @@ __credits__ = [
     'Phil Swanson <swanson.p@gmail.com>',
     'Matthew Tai <taim@google.com>',
     'Diogo Terror <me@diogoterror.com>',
+    'William Vambenepe <vbp@google.com>',
+    'Daniel Vetter <lord_fright@yahoo.com>',
     'Paul Wais <pwais@yelp.com>',
+    'Qiu Wei <qiuwei.cw@alibaba-inc.com>',
     'Derek Wilson <jderekwilson@gmail.com>',
     'Tao Yu <taoyu@yelp.com>',
     'Andrea Zonca <andrea.zonca@gmail.com>',
 ]
 
-__version__ = '0.5.1'
+__version__ = '0.6.3'
