@@ -1,4 +1,5 @@
-# Copyright 2009-2016 Yelp and Contributors
+# Copyright 2009-2017 Yelp and Contributors
+# Copyright 2018 Yelp
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -20,54 +21,32 @@ import inspect
 import itertools
 import json
 import logging
+import os.path
 import sys
-from optparse import OptionGroup
 
 # don't use relative imports, to allow this script to be invoked as __main__
+from mrjob.cat import decompress
 from mrjob.conf import combine_dicts
+from mrjob.conf import combine_lists
+from mrjob.launch import MRJobLauncher
+from mrjob.launch import _im_func
+from mrjob.launch import _READ_ARGS_FROM_SYS_ARGV
+from mrjob.options import _add_step_args
+from mrjob.options import _print_help_for_steps
 from mrjob.protocol import JSONProtocol
 from mrjob.protocol import RawValueProtocol
-from mrjob.launch import MRJobLauncher
-from mrjob.launch import _READ_ARGS_FROM_SYS_ARGV
-from mrjob.step import MRStep
-from mrjob.step import _JOB_STEP_FUNC_PARAMS
 from mrjob.py2 import integer_types
 from mrjob.py2 import string_types
-from mrjob.util import read_input
+from mrjob.step import MRStep
+from mrjob.step import SparkStep
+from mrjob.step import _JOB_STEP_FUNC_PARAMS
+from mrjob.util import expand_path
+from mrjob.util import to_lines
 
 
 log = logging.getLogger(__name__)
 
 
-# jobconf options for implementing SORT_VALUES
-_SORT_VALUES_JOBCONF = {
-    'stream.num.map.output.key.fields': 2,
-    'mapred.text.key.partitioner.options': '-k1,1',
-    # Hadoop's defaults for these actually work fine; we just want to
-    # prevent interference from mrjob.conf.
-    'mapred.output.key.comparator.class': None,
-    'mapred.text.key.comparator.options': None,
-}
-
-# partitioner for sort_values
-_SORT_VALUES_PARTITIONER = \
-    'org.apache.hadoop.mapred.lib.KeyFieldBasedPartitioner'
-
-
-def _im_func(f):
-    """Wrapper to get at the underlying function belonging to a method.
-
-    Python 2 is slightly different because classes have "unbound methods"
-    which wrap the underlying function, whereas on Python 3 they're just
-    functions. (Methods work the same way on both versions.)
-    """
-    # "im_func" is the old Python 2 name for __func__
-    if hasattr(f, '__func__'):
-        return f.__func__
-    else:
-        return f
-
-
 class UsageError(Exception):
     pass
 
@@ -76,9 +55,8 @@ class MRJob(MRJobLauncher):
     """The base class for all MapReduce jobs. See :py:meth:`__init__`
     for details."""
 
-    # inline can be the default because we have the class object in the same
-    # process as the launcher
-    _DEFAULT_RUNNER = 'inline'
+    # script path is whatever file our subclass of MRJob is in
+    _FIRST_ARG_IS_SCRIPT_PATH = False
 
     def __init__(self, args=None):
         """Entry point for running your job from other Python code.
@@ -99,11 +77,13 @@ class MRJob(MRJobLauncher):
         """
         super(MRJob, self).__init__(self.mr_job_script(), args)
 
+        self._warned_about_parse_output_line = False
+
     @classmethod
     def _usage(cls):
-        return "usage: %prog [options] [input files]"
+        return "usage: %(prog)s [options] [input files]"
 
-    ### Defining one-step jobs ###
+    ### Defining one-step streaming jobs ###
 
     def mapper(self, key, value):
         """Re-define this to define the mapper for a one-step job.
@@ -210,6 +190,17 @@ class MRJob(MRJobLauncher):
         """
         raise NotImplementedError
 
+    def mapper_raw(self, input_path, input_uri):
+        """Re-define this to make Hadoop pass one input file to each
+        mapper.
+
+        :param input_path: a local path that the input file has been copied to
+        :param input_uri: the URI of the input file on HDFS, S3, etc
+
+        .. versionadded:: 0.6.3
+        """
+        raise NotImplementedError
+
     def reducer_init(self):
         """Re-define this to define an action to run before the reducer
         processes any input.
@@ -310,6 +301,18 @@ class MRJob(MRJobLauncher):
         """
         raise NotImplementedError
 
+    ### Defining one-step Spark jobs ###
+
+    def spark(self, input_path, output_path):
+        """Re-define this with Spark code to run. You can read input
+        with *input_path* and output with *output_path*.
+        """
+        raise NotImplementedError
+
+    def spark_args(self):
+        """Redefine this to pass custom arguments to Spark."""
+        return []
+
     ### Defining multi-step jobs ###
 
     def steps(self):
@@ -322,40 +325,43 @@ class MRJob(MRJobLauncher):
         re-defined. For example::
 
             def steps(self):
-                return [self.mr(mapper=self.transform_input,
-                                reducer=self.consolidate_1),
-                        self.mr(reducer_init=self.log_mapper_init,
-                                reducer=self.consolidate_2)]
+                return [MRStep(mapper=self.transform_input,
+                               reducer=self.consolidate_1),
+                        MRStep(reducer_init=self.log_mapper_init,
+                               reducer=self.consolidate_2)]
 
-        :return: a list of steps constructed with :py:meth:`mr`
+        :return: a list of steps constructed with
+                 :py:class:`~mrjob.step.MRStep` or other classes in
+                 :py:mod:`mrjob.step`.
         """
         # only include methods that have been redefined
         kwargs = dict(
             (func_name, getattr(self, func_name))
-            for func_name in _JOB_STEP_FUNC_PARAMS
+            for func_name in _JOB_STEP_FUNC_PARAMS + ('spark',)
             if (_im_func(getattr(self, func_name)) is not
                 _im_func(getattr(MRJob, func_name))))
 
+        # special case for spark()
+        # TODO: support jobconf as well
+        if 'spark' in kwargs:
+            if sorted(kwargs) != ['spark']:
+                raise ValueError(
+                    "Can't mix spark() and streaming functions")
+            return [SparkStep(
+                spark=kwargs['spark'],
+                spark_args=self.spark_args())]
+
         # MRStep takes commands as strings, but the user defines them in the
         # class as functions that return strings, so call the functions.
         updates = {}
         for k, v in kwargs.items():
-            if k.endswith('_cmd'):
+            if k.endswith('_cmd') or k.endswith('_pre_filter'):
                 updates[k] = v()
 
         kwargs.update(updates)
 
         return [MRStep(**kwargs)]
 
-    @classmethod
-    def mr(cls, **kwargs):
-        """A deprecated wrapper for :py:class:`~mrjob.step.MRStep`.
-        """
-        log.warning('mr() is deprecated and will be removed in v0.6.0.'
-                    ' Use mrjob.step.MRStep directly instead.')
-
-        return MRStep(**kwargs)
-
     def increment_counter(self, group, counter, amount=1):
         """Increment a counter in Hadoop streaming by printing to stderr.
 
@@ -444,6 +450,9 @@ class MRJob(MRJobLauncher):
         elif self.options.run_reducer:
             self.run_reducer(self.options.step_num)
 
+        elif self.options.run_spark:
+            self.run_spark(self.options.step_num)
+
         else:
             super(MRJob, self).execute()
 
@@ -454,41 +463,58 @@ class MRJob(MRJobLauncher):
         :rtype: :py:class:`mrjob.runner.MRJobRunner`
         """
         bad_words = (
-            '--steps', '--mapper', '--reducer', '--combiner', '--step-num')
+            '--steps', '--mapper', '--reducer', '--combiner', '--step-num',
+            '--spark')
         for w in bad_words:
             if w in sys.argv:
                 raise UsageError("make_runner() was called with %s. This"
                                  " probably means you tried to use it from"
                                  " __main__, which doesn't work." % w)
 
-        # support inline runner when running from the MRJob itself
-        from mrjob.inline import InlineMRJobRunner
-
-        if self.options.runner == 'inline':
-            return InlineMRJobRunner(mrjob_cls=self.__class__,
-                                     **self.inline_job_runner_kwargs())
-
         return super(MRJob, self).make_runner()
 
+    def _runner_class(self):
+        """Runner class as indicated by ``--runner``. Defaults to ``'inline'``.
+        """
+        if not self.options.runner or self.options.runner == 'inline':
+            from mrjob.inline import InlineMRJobRunner
+            return InlineMRJobRunner
+
+        else:
+            return super(MRJob, self)._runner_class()
+
+    def _runner_kwargs(self):
+        """If we're building an inline runner, include mrjob_cls in kwargs."""
+        kwargs = super(MRJob, self)._runner_kwargs()
+
+        if self._runner_class().alias == 'inline':
+            kwargs = dict(mrjob_cls=self.__class__, **kwargs)
+
+        return kwargs
+
+    def _get_step(self, step_num, expected_type):
+        """Helper for run_* methods"""
+        steps = self.steps()
+        if not 0 <= step_num < len(steps):
+            raise ValueError('Out-of-range step: %d' % step_num)
+        step = steps[step_num]
+        if not isinstance(step, expected_type):
+            raise TypeError('Step %d is not a %s', expected_type.__name__)
+        return step
+
     def run_mapper(self, step_num=0):
         """Run the mapper and final mapper action for the given step.
 
         :type step_num: int
         :param step_num: which step to run (0-indexed)
 
-        If --no-strict-protocols is set, and we encounter a line that can't
-        be decoded by our input protocol or a tuple that can't be encoded
-        by our output protocol, we'll increment a counter rather than
-        raising an exception. This will be going away in v0.6.0.
-
         Called from :py:meth:`run`. You'd probably only want to call this
         directly from automated tests.
         """
-        steps = self.steps()
-        if not 0 <= step_num < len(steps):
-            raise ValueError('Out-of-range step: %d' % step_num)
-        step = steps[step_num]
+        step = self._get_step(step_num, MRStep)
+
         mapper = step['mapper']
+        mapper_raw = step['mapper_raw']
         mapper_init = step['mapper_init']
         mapper_final = step['mapper_final']
 
@@ -499,10 +525,18 @@ class MRJob(MRJobLauncher):
             for out_key, out_value in mapper_init() or ():
                 write_line(out_key, out_value)
 
-        # run the mapper on each line
-        for key, value in read_lines():
-            for out_key, out_value in mapper(key, value) or ():
+        if mapper_raw:
+            # pass input path and uri to mapper_raw()
+            if len(self.options.args) != 2:
+                raise ValueError('Wrong number of args')
+            input_path, input_uri = self.options.args
+            for out_key, out_value in mapper_raw(input_path, input_uri) or ():
                 write_line(out_key, out_value)
+        else:
+            # run the mapper on each line
+            for key, value in read_lines():
+                for out_key, out_value in mapper(key, value) or ():
+                    write_line(out_key, out_value)
 
         if mapper_final:
             for out_key, out_value in mapper_final() or ():
@@ -522,10 +556,8 @@ class MRJob(MRJobLauncher):
         Called from :py:meth:`run`. You'd probably only want to call this
         directly from automated tests.
         """
-        steps = self.steps()
-        if not 0 <= step_num < len(steps):
-            raise ValueError('Out-of-range step: %d' % step_num)
-        step = steps[step_num]
+        step = self._get_step(step_num, MRStep)
+
         reducer = step['reducer']
         reducer_init = step['reducer_init']
         reducer_final = step['reducer_final']
@@ -567,10 +599,8 @@ class MRJob(MRJobLauncher):
         Called from :py:meth:`run`. You'd probably only want to call this
         directly from automated tests.
         """
-        steps = self.steps()
-        if not 0 <= step_num < len(steps):
-            raise ValueError('Out-of-range step: %d' % step_num)
-        step = steps[step_num]
+        step = self._get_step(step_num, MRStep)
+
         combiner = step['combiner']
         combiner_init = step['combiner_init']
         combiner_final = step['combiner_final']
@@ -598,6 +628,24 @@ class MRJob(MRJobLauncher):
             for out_key, out_value in combiner_final() or ():
                 write_line(out_key, out_value)
 
+    def run_spark(self, step_num):
+        """Run the Spark code for the given step.
+
+        :type step_num: int
+        :param step_num: which step to run (0-indexed)
+
+        Called from :py:meth:`run`. You'd probably only want to call this
+        directly from automated tests.
+        """
+        step = self._get_step(step_num, SparkStep)
+
+        if len(self.options.args) != 2:
+            raise ValueError('Wrong number of args')
+        input_path, output_path = self.options.args
+
+        spark_method = step.spark
+        spark_method(input_path, output_path)
+
     def show_steps(self):
         """Print information about how many steps there are, and whether
         they contain a mapper or reducer. Job runners (see
@@ -642,10 +690,16 @@ class MRJob(MRJobLauncher):
         - If path is ``-``, read from STDIN.
         - Recursively read all files in a directory
         """
-        paths = self.args or ['-']
+        paths = self.options.args or ['-']
+
         for path in paths:
-            for line in read_input(path, stdin=self.stdin):
-                yield line
+            if path == '-':
+                for line in self.stdin:
+                    yield line
+            else:
+                with open(path, 'rb') as f:
+                    for line in to_lines(decompress(f, path)):
+                        yield line
 
     def _wrap_protocols(self, step_num, step_type):
         """Pick the protocol classes to use for reading and writing
@@ -668,29 +722,12 @@ class MRJob(MRJobLauncher):
 
         def read_lines():
             for line in self._read_input():
-                try:
-                    key, value = read(line.rstrip(b'\r\n'))
-                    yield key, value
-                except Exception as e:
-                    # the strict_protocols option has to default to None
-                    # because it's used by runners, so treat None as true
-                    if self.options.strict_protocols is not False:
-                        raise
-                    else:
-                        self.increment_counter(
-                            'Undecodable input', e.__class__.__name__)
+                key, value = read(line.rstrip(b'\r\n'))
+                yield key, value
 
         def write_line(key, value):
-            try:
-                self.stdout.write(write(key, value))
-                self.stdout.write(b'\n')
-            except Exception as e:
-                # None counts as true, see above
-                if self.options.strict_protocols is not False:
-                    raise
-                else:
-                    self.increment_counter(
-                        'Unencodable output', e.__class__.__name__)
+            self.stdout.write(write(key, value))
+            self.stdout.write(b'\n')
 
         return read_lines, write_line
 
@@ -707,16 +744,16 @@ class MRJob(MRJobLauncher):
         mapping = {}
         script_step_num = 0
         for i, step in enumerate(steps_desc):
-            if 'mapper' in step:
-                if step['mapper']['type'] == 'script':
-                    k = self._step_key(i, 'mapper')
-                    mapping[k] = script_step_num
-                    script_step_num += 1
-            if 'reducer' in step:
-                if step['reducer']['type'] == 'script':
-                    k = self._step_key(i, 'reducer')
-                    mapping[k] = script_step_num
-                    script_step_num += 1
+
+            if 'mapper' in step and step['mapper']['type'] == 'script':
+                k = self._step_key(i, 'mapper')
+                mapping[k] = script_step_num
+                script_step_num += 1
+
+            if 'reducer' in step and step['reducer']['type'] == 'script':
+                k = self._step_key(i, 'reducer')
+                mapping[k] = script_step_num
+                script_step_num += 1
 
         return mapping
 
@@ -794,63 +831,41 @@ class MRJob(MRJobLauncher):
 
     ### Command-line arguments ###
 
-    def configure_options(self):
-        super(MRJob, self).configure_options()
+    def configure_args(self):
+        """Define arguments for this script. Called from :py:meth:`__init__()`.
 
-        # To run mappers or reducers
-        self.mux_opt_group = OptionGroup(
-            self.option_parser, 'Running specific parts of the job')
-        self.option_parser.add_option_group(self.mux_opt_group)
+        Re-define to define custom command-line arguments or pass
+        through existing ones::
 
-        self.mux_opt_group.add_option(
-            '--mapper', dest='run_mapper', action='store_true', default=False,
-            help='run a mapper')
+            def configure_args(self):
+                super(MRYourJob, self).configure_args()
 
-        self.mux_opt_group.add_option(
-            '--combiner', dest='run_combiner', action='store_true',
-            default=False, help='run a combiner')
+                self.add_passthru_arg(...)
+                self.add_file_arg(...)
+                self.pass_arg_through(...)
+                ...
+        """
+        super(MRJob, self).configure_args()
 
-        self.mux_opt_group.add_option(
-            '--reducer', dest='run_reducer', action='store_true',
-            default=False, help='run a reducer')
-
-        self.mux_opt_group.add_option(
-            '--step-num', dest='step_num', type='int', default=0,
-            help='which step to execute (default is 0)')
-
-        # To describe the steps
-        self.mux_opt_group.add_option(
-            '--steps', dest='show_steps', action='store_true', default=False,
-            help=('print the mappers, combiners, and reducers that this job'
-                  ' defines'))
-
-    def all_option_groups(self):
-        return super(MRJob, self).all_option_groups() + (self.mux_opt_group,)
+        _add_step_args(self.arg_parser)
 
     def is_task(self):
-        """True if this is a mapper, combiner, or reducer.
+        """True if this is a mapper, combiner, reducer, or Spark script.
 
-        This is mostly useful inside :py:meth:`load_options`, to disable
-        loading options when we aren't running inside Hadoop Streaming.
+        This is mostly useful inside :py:meth:`load_args`, to disable
+        loading args when we aren't running inside Hadoop.
         """
         return (self.options.run_mapper or
                 self.options.run_combiner or
-                self.options.run_reducer)
+                self.options.run_reducer or
+                self.options.run_spark)
 
-    def _process_args(self, args):
-        """mrjob.launch takes the first arg as the script path, but mrjob.job
-        uses all args as input files. This method determines the behavior:
-        MRJob uses all args as input files.
-        """
-        self.args = args
-
-    def _help_main(self):
-        self.option_parser.option_groups = [
-            self.mux_opt_group,
-            self.proto_opt_group,
-        ]
-        self.option_parser.print_help()
-        sys.exit(0)
+    def _print_help(self, options):
+        """Implement --help --steps"""
+        if options.show_steps:
+            _print_help_for_steps()
+        else:
+            super(MRJob, self)._print_help(options)
 
     ### protocols ###
 
@@ -919,15 +934,29 @@ class MRJob(MRJobLauncher):
     #: See :py:data:`mrjob.protocol` for the full list of protocols.
     OUTPUT_PROTOCOL = JSONProtocol
 
+    def parse_output(self, chunks):
+        """Parse the final output of this MRJob (as a stream of byte chunks)
+        into a stream of ``(key, value)``.
+        """
+        read = self.output_protocol().read
+
+        for line in to_lines(chunks):
+            yield read(line)
+
     def parse_output_line(self, line):
         """
         Parse a line from the final output of this MRJob into
-        ``(key, value)``. Used extensively in tests like this::
+        ``(key, value)``.
 
-            runner.run()
-            for line in runner.stream_output():
-                key, value = mr_job.parse_output_line(line)
+        .. deprecated:: 0.6.0
+
+           Use :py:meth:`parse_output` instead.
         """
+        if not self._warned_about_parse_output_line:
+            log.warning('parse_output_line() is deprecated and will be removed'
+                        ' in v0.7.0; use parse_output() instead.')
+            self._warned_about_parse_output_line = True
+
         return self.output_protocol().read(line)
 
     ### Hadoop Input/Output Formats ###
@@ -972,6 +1001,55 @@ class MRJob(MRJobLauncher):
         """
         return self.HADOOP_OUTPUT_FORMAT
 
+    ### Libjars ###
+
+    #: Optional list of paths of jar files to run our job with using Hadoop's
+    #: ``-libjar`` option.
+    #:
+    #: ``~`` and environment variables
+    #: in paths be expanded, and relative paths will be interpreted as
+    #: relative to the directory containing the script (not the current
+    #: working directory).
+    #:
+    #: If you require more sophisticated behavior, try overriding
+    #: :py:meth:`libjars`.
+    #:
+    #: .. versionadded:: 0.5.3
+    LIBJARS = []
+
+    def libjars(self):
+        """Optional list of paths of jar files to run our job with using
+        Hadoop's ``-libjar`` option. Normally setting :py:attr:`LIBJARS`
+        is sufficient.
+
+        By default, this combines :option:`libjars` options from the command
+        lines with :py:attr:`LIBJARS`, with command line arguments taking
+        precedence. Paths from :py:attr:`LIBJARS` are interpreted as relative
+        to the the directory containing the script (paths from the
+        command-line are relative to the current working directory).
+
+        Note that ``~`` and environment variables in paths will always be
+        expanded by the job runner (see :mrjob-opt:`libjars`).
+
+        .. versionadded:: 0.5.3
+        """
+        script_dir = os.path.dirname(self.mr_job_script())
+
+        paths_from_libjars = []
+
+        # libjar paths will eventually be combined with combine_path_lists,
+        # which will expand environment variables. We don't want to assume
+        # a path like $MY_DIR/some.jar is always relative ($MY_DIR could start
+        # with /), but we also don't want to expand environment variables
+        # prematurely.
+        for path in self.LIBJARS or []:
+            if os.path.isabs(expand_path(path)):
+                paths_from_libjars.append(path)
+            else:
+                paths_from_libjars.append(os.path.join(script_dir, path))
+
+        return combine_lists(paths_from_libjars, self.options.libjars)
+
     ### Partitioning ###
 
     #: Optional Hadoop partitioner class to use to determine how mapper
@@ -985,18 +1063,12 @@ class MRJob(MRJobLauncher):
         """Optional Hadoop partitioner class to use to determine how mapper
         output should be sorted and distributed to reducers.
 
-        By default, returns whatever is passed to :option:`--partitioner`,
-        or if that option isn't used, :py:attr:`PARTITIONER`, or if that
-        isn't set, and :py:attr:`SORT_VALUES` is true, it's set to
-        ``'org.apache.hadoop.mapred.lib.KeyFieldBasedPartitioner'``.
+        By default, returns :py:attr:`PARTITIONER`.
 
         You probably don't need to re-define this; it's just here for
         completeness.
         """
-        return (self.options.partitioner or
-                self.PARTITIONER or
-                ('org.apache.hadoop.mapred.lib.KeyFieldBasedPartitioner' if
-                 self.SORT_VALUES else None))
+        return self.PARTITIONER
 
     ### Jobconf ###
 
@@ -1018,11 +1090,6 @@ class MRJob(MRJobLauncher):
         lines with :py:attr:`JOBCONF`, with command line arguments taking
         precedence.
 
-        If :py:attr:`SORT_VALUES` is set, we also set these jobconf values::
-
-            stream.num.map.output.key.fields=2
-            mapred.text.key.partitioner.options=k1,1
-
         We also blank out ``mapred.output.key.comparator.class``
         and ``mapred.text.key.comparator.options`` to prevent interference
         from :file:`mrjob.conf`.
@@ -1051,44 +1118,14 @@ class MRJob(MRJobLauncher):
 
                 return mrjob.conf.combine_dicts(orig_jobconf, custom_jobconf)
         """
-
-        # deal with various forms of bad behavior by users
+        # combine job and runner jobconf
         unfiltered_jobconf = combine_dicts(self.JOBCONF, self.options.jobconf)
-        filtered_jobconf = {}
 
-        def format_hadoop_version(v_float):
-            if v_float >= 1.0:
-                # e.g. 1.0
-                return '%.1f' % v_float
-            else:
-                # e.g. 0.20
-                return '%.2f' % v_float
-
-        for key in unfiltered_jobconf:
-            unfiltered_val = unfiltered_jobconf[key]
-            filtered_val = unfiltered_val
-
-            # boolean values need to be lowercased
-            if isinstance(unfiltered_val, bool):
-                if unfiltered_val:
-                    filtered_val = 'true'
-                else:
-                    filtered_val = 'false'
-
-            # TODO: why would a jobconf variable be named 'hadoop_version'?
-            # hadoop_version should be a string
-            elif (key == 'hadoop_version' and
-                    isinstance(unfiltered_val, float)):
-                log.warning('hadoop_version should be a string, not %s' %
-                            unfiltered_val)
-                filtered_val = format_hadoop_version(unfiltered_val)
-            filtered_jobconf[key] = filtered_val
-
-        if self.SORT_VALUES:
-            filtered_jobconf = combine_dicts(
-                _SORT_VALUES_JOBCONF, filtered_jobconf)
-
-        return filtered_jobconf
+        # turn booleans into the Java equivalent ("false", not "False")
+        return {
+            k: json.dumps(v) if not isinstance(v, string_types) else v
+            for k, v in unfiltered_jobconf.items() if v is not None
+        }
 
     ### Secondary Sort ###
 
@@ -1109,11 +1146,15 @@ class MRJob(MRJobLauncher):
     #: you're using.
     #:
     #: See :py:meth:`jobconf()` and :py:meth:`partitioner()` for more about
-    #: how this works.
-    #:
-    #: .. versionadded:: 0.4.1
     SORT_VALUES = None
 
+    def sort_values(self):
+        """A method that by default, just returns the value of
+        :py:attr:`SORT_VALUES`. Mostly exists for the sake
+        of consistency, but you could override it if you wanted to make
+        secondary sort configurable."""
+        return self.SORT_VALUES
+
 
 if __name__ == '__main__':
     MRJob.run()
