@@ -1,4 +1,7 @@
-# Copyright 2009-2010 Yelp
+# Copyright 2009-2013 Yelp, David Marin
+# Copyright 2015 Yelp
+# Copyright 2017 Yelp
+# Copyright 2018 Contributors
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -11,32 +14,41 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
-"""A simple wrapper for retrying when we get transient errors."""
+"""Wrappers for gracefully retrying on error."""
 import logging
 import time
 
-log = logging.getLogger('mrjob.retry')
+log = logging.getLogger(__name__)
+
 
 class RetryWrapper(object):
-    """This class can wrap any object. The wrapped object will behave like
+    """Handle transient errors, with configurable backoff.
+
+    This class can wrap any object. The wrapped object will behave like
     the original one, except that if you call a function and it raises a
     retriable exception, we'll back off for a certain number of seconds
     and call the function again, until it succeeds or we get a non-retriable
     exception.
     """
+    # TODO: this doesn't correctly handle object properties or wrapping
+    # functions.
     def __init__(self, wrapped, retry_if, backoff=15, multiplier=1.5,
                  max_tries=10):
         """
         Wrap the given object
 
-        :param wrapped: the object to wrap. this should work for functions as well as regular objects (essentially, we're wrapping ``__call__``)
-        :param retry_if: a method that takes an exception, and returns whether we should retry
+        :param wrapped: the object to wrap
+        :param retry_if: a method that takes an exception, and returns whether
+                         we should retry
         :type backoff: float
-        :param backoff: the number of seconds to wait the first time we get a retriable error.
+        :param backoff: the number of seconds to wait the first time we get a
+                        retriable error
         :type multiplier: float
-        :param multiplier: if we retry multiple times, the amount to multiply the backoff time by every time we get an error
+        :param multiplier: if we retry multiple times, the amount to multiply
+                           the backoff time by every time we get an error
         :type max_tries: int
-        :param max_tries: how many tries we get. ``0`` means to keep trying forever
+        :param max_tries: how many tries we get. ``0`` means to keep trying
+                          forever
         """
         self.__wrapped = wrapped
 
@@ -68,11 +80,13 @@ class RetryWrapper(object):
             backoff = self.__backoff
             tries = 0
 
-            while (not self.__max_tries or tries < self.__max_tries):
+            while not self.__max_tries or tries < self.__max_tries:
                 try:
                     return f(*args, **kwargs)
-                except Exception, ex:
-                    if self.__retry_if(ex):
+                except Exception as ex:
+                    if (self.__retry_if(ex) and
+                        (tries < self.__max_tries - 1 or
+                         not self.__max_tries)):
                         log.info('Got retriable error: %r' % ex)
                         log.info('Backing off for %.1f seconds' % backoff)
                         time.sleep(backoff)
@@ -82,5 +96,6 @@ class RetryWrapper(object):
                         raise
 
         # pretend to be the original function
-        call_and_maybe_retry.__name__ == f.__name__
+        if hasattr(f, '__name__'):
+            call_and_maybe_retry.__name__ = f.__name__
         return call_and_maybe_retry
