@@ -1,6 +1,7 @@
 # Copyright 2012 Yelp and Contributors
 # Copyright 2013 David Marin and Lyft
 # Copyright 2014-2016 Yelp and Contributors
+# Copyright 2017 Yelp
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -35,6 +36,7 @@ manager classes.
 """
 import itertools
 import logging
+import os
 import os.path
 import posixpath
 import re
@@ -47,7 +49,7 @@ from mrjob.util import expand_path
 log = logging.getLogger(__name__)
 
 
-_SUPPORTED_TYPES = ('archive', 'file')
+_SUPPORTED_TYPES = ('archive', 'dir', 'file')
 
 
 _SETUP_CMD_RE = re.compile(
@@ -55,7 +57,8 @@ _SETUP_CMD_RE = re.compile(
     r'(?P<double_quoted>"([^"\\]|\\.)*")|'
     r'(?P<hash_path>'
         r'(?P<path>([A-Za-z][A-Za-z0-9\.-]*://([^\'"\s\\]|\\.)+)|'  # noqa
-            r'([^\'":=\s\\]|\\.)+)'  # noqa
+            r'([^\'":=\s\\]|\\.)*([^\'":=\s\\/]|\\.))'  # noqa
+            r'(?P<path_slash>/)?'
         r'#(?P<name>([^\'":;><|=/#\s\\]|\\.)*)'
             r'(?P<name_slash>/)?)|'
     r'(?P<unquoted>([^\'":=\s\\]|\\.)+)|'
@@ -86,8 +89,16 @@ def parse_setup_cmd(cmd):
     remote system. The trailing slash will *also* be kept as part of the
     original command.
 
+    If *path* is followed by a trailing slash, that indicates *path* is a
+    directory and should be tarballed and later unarchived into a directory
+    on the remote system. The trailing slash will also be kept as part of
+    the original command. You may optionally include a slash after *name* as
+    well (this will only result in a single slash in the final command).
+
+    .. versionadded:: 0.5.8 support for directories (above)
+
     Parsed hash paths are dicitionaries with the keys ``path``, ``name``, and
-    ``type`` (either ``'file'`` or ``'archive'``).
+    ``type`` (either ``'file'``, ``'archive'``, or ``'dir'``).
 
     Most of the time, this function will just do what you expect. Rules for
     finding hash paths:
@@ -127,11 +138,20 @@ def parse_setup_cmd(cmd):
             else:
                 tokens.append(keep_as_is)
         elif m.group('hash_path'):
+            if m.group('path_slash'):
+                token_type = 'dir'
+            elif m.group('name_slash'):
+                token_type = 'archive'
+            else:
+                token_type = 'file'
+
             tokens.append({
                 'path': _resolve_path(m.group('path')),
                 'name': m.group('name') or None,
-                'type': 'archive' if m.group('name_slash') else 'file'})
-            if m.group('name_slash'):
+                'type': token_type
+            })
+
+            if m.group('path_slash') or m.group('name_slash'):
                 tokens.append('/')
         elif m.group('error'):
             # these match the error messages from shlex.split()
@@ -185,17 +205,21 @@ def parse_legacy_hash_path(type, path, must_name=None):
     if '#' in path:
         path, name = path.split('#', 1)
 
-        # allow a slash after the name of an archive because that's
+        # allow a slash after the name of an archive or dir because that's
         # the new-way of specifying archive paths
-        if name.endswith('/') and type == 'archive':
-            name = name[:-1]
+        if type in ('archive', 'dir'):
+            name = name.rstrip('/' + os.sep)
 
         if '/' in name or '#' in name:
             raise ValueError(
                 'Bad path %r; name must not contain # or /' % (path,))
     else:
         if must_name:
-            name = os.path.basename(path)
+            if type == 'dir':
+                # handle trailing slash on directory names
+                name = os.path.basename(path.rstrip('/' + os.sep))
+            else:
+                name = os.path.basename(path)
         else:
             name = None
 
@@ -212,12 +236,14 @@ def parse_legacy_hash_path(type, path, must_name=None):
     return {'path': path, 'name': name, 'type': type}
 
 
-def name_uniquely(path, names_taken=(), proposed_name=None):
+def name_uniquely(path, names_taken=(), proposed_name=None, unhide=False):
     """Come up with a unique name for *path*.
 
     :param names_taken: a dictionary or set of names not to use.
     :param proposed_name: name to use if it is not taken. If this is not set,
                           we propose a name based on the filename.
+    :param unhide: make sure final name doesn't start with periods or
+                   underscores
 
     If the proposed name is taken, we add a number to the end of the
     filename, keeping the extension the same. For example:
@@ -226,19 +252,29 @@ def name_uniquely(path, names_taken=(), proposed_name=None):
     'foo-1.tar.gz'
     """
     if not proposed_name:
-        proposed_name = os.path.basename(path) or '_'
+        proposed_name = os.path.basename(path.rstrip('/' + os.sep))
 
-    if proposed_name not in names_taken:
-        return proposed_name
+    if unhide:
+        proposed_name = proposed_name.lstrip('.').lstrip('_')
 
-    dot_idx = proposed_name.find('.', 1)
+    # don't treat initial . as part of file extension (but unhide
+    # would have already handled this)
+    dot_idx = proposed_name.find('.', 0 if unhide else 1)
     if dot_idx == -1:
         prefix, suffix = proposed_name, ''
     else:
         prefix, suffix = proposed_name[:dot_idx], proposed_name[dot_idx:]
 
+    if prefix and proposed_name not in names_taken:
+        return proposed_name
+
     for i in itertools.count(1):
-        name = '%s-%d%s' % (prefix, i, suffix)
+        if prefix:
+            name = '%s-%d%s' % (prefix, i, suffix)
+        else:
+            # if no prefix is left (due to empty filename or unhiding)
+            # just use numbers; don't start filenames with '-'
+            name = '%d%s' % (i, suffix)
         if name not in names_taken:
             return name
 
@@ -276,7 +312,10 @@ class UploadDirManager(object):
             return path
 
         if path not in self._path_to_name:
-            name = name_uniquely(path, names_taken=self._names_taken)
+            # use unhide so that input files won't be hidden from Hadoop,
+            # see #1200
+            name = name_uniquely(
+                path, names_taken=self._names_taken, unhide=True)
             self._names_taken.add(name)
             self._path_to_name[path] = name
 
@@ -315,7 +354,9 @@ class WorkingDirManager(object):
     If you wish, you may assign multiple names to the same file, or add
     a path as both a file and an archive (though not mapped to the same name).
     """
-    _SUPPORTED_TYPES = _SUPPORTED_TYPES
+    # dirs are not supported directly; runners need to archive them
+    # and add that archive
+    _SUPPORTED_TYPES = ('archive', 'file')
 
     def __init__(self):
         # map from paths added without a name to None or lazily chosen name
@@ -431,12 +472,5 @@ class WorkingDirManager(object):
 
     def _check_type(self, type):
         if type not in self._SUPPORTED_TYPES:
-            raise ValueError('bad path type %r, must be one of %s' % (
+            raise ValueError('bad path type %r, must be one of: %s' % (
                 type, ', '.join(sorted(self._SUPPORTED_TYPES))))
-
-
-class BootstrapWorkingDirManager(WorkingDirManager):
-    """Manage the working dir for the master bootstrap script. Identical
-    to :py:class:`WorkingDirManager` except that it doesn't support archives.
-    """
-    _SUPPORTED_TYPES = ('file',)
